<?php
/**
* MySQL
*
*/

class MySQL
{
	/**
	* identifiant de connexion au serveur mysql
	* @var false|integer
	*/
	private $cid = false;

	/**
	* indique si une transaction est valide
	* @var bool|integer
	*/
	private $transValid = true;

	/**
	* serveur mysql
	* @var string
	*/
	private $host;

	/**
	* nom de la base
	* @var string
	*/
	private $nomBase;

	/**
	* login de connexion
	* @var string
	*/
	private $user;

	/**
	* mot de passe de connexion
	* @var string
	*/
	private $pass;

	/**
	* afficher les requtes dans la trace
	* @var bool
	*/
	private $debug = false;

	/**
	* message d'erreur pour les transactions
	* @var string
	*/
	private $reqEnErreur = '';

	/**
	* trace
	* @var Trace
	*/
	protected $trace;


	/**
	* initialisation des variables de la classe
	* @param array $param tableau de paramtres
	*/
	protected function __construct(&$param)
	{
		$this->trace = Trace::singleton();
		//rcupration des paramtres
		$this->host = $param['host'];
		$this->nomBase = $param['nomBase'];
		$this->user = $param['user'];
		$this->pass = $param['pass'];
		$this->debug = false;
		if(isset($param['debug']) && $param['debug'])
			$this->debug = true;
	}

	/**
	* destructeur
	* dconnexion si non ralise
	*/
	protected function __destruct()
	{
		//On vrifie que la connexion est bien ferme
		if($this->cid)
			$this->deconnect();
	}

	/**
	* connexion  la base
	*/
	protected function connect()
	{
		$this->cid = mysql_connect($this->host, $this->user, $this->pass);
		mysql_select_db($this->nomBase, $this->cid);
	}

	/**
	* dconnexion de la base
	*/
	protected function deconnect()
	{
		mysql_close($this->cid);
		$this->cid = false;
	}

	/**
	* excuter une requte sur la base
	* @param string $req requte SQL
	* @return integer|true retourne une ressource de rsultat ou true sinon dclenche une erreur
	*/
	protected function requete($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		$resultat = mysql_query($req, $this->cid);
		if($resultat === false)
		{
			$numErreur = mysql_errno($this->cid);
			$this->deconnect();
			trigger_error('Requte errone ou droits insuffisants de l\'utilisateur pour l\'effectuer : (erreur n'.$numErreur.') '.$req, E_USER_ERROR);
		}
		else
			return $resultat;
	}

	/**
	* requte UPDATE
	*  utiliser si l'update concerne au moins
	* une ligne obligatoirement prsente
	* si aucune ligne  mettre  jour trouve : erreur dclenche
	* @param string $req requte SQL avec UPDATE
	* @return integer|true retourne une ressource de rsultat ou true sinon dclenche une erreur
	*/
	protected function requeteUpdate($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		$resultat = mysql_query($req, $this->cid);

		//au moins une ligne prsente?
		if(!$this->mysql_matched_rows($this->cid))
		{
			$this->deconnect();
			trigger_error('Requte avec un update d\'une ligne non existante requise : '.$req, E_USER_ERROR);
		}
		//pb contrainte de cl?
		elseif($resultat === false)
		{
			$numErreur = mysql_errno($this->cid);
			if($numErreur==1216 || $numErreur==1217 || $numErreur==1451 || $numErreur==1452)
			{
				$this->deconnect();
				trigger_error('Requte errone : une contrainte de cl trangre choue : '.$req, E_USER_ERROR);
			}
			else
			{
				$this->deconnect();
				trigger_error('Requte errone ou droits insuffisants de l\'utilisateur pour l\'effectuer : (erreur n'.$numErreur.') '.$req, E_USER_ERROR);
			}
		}
		else
			return $resultat;
	}

	/**
	* excuter une requte de suppression sur la base
	* retourne true
	* ou false si contrainte de cl trangre empche la suppression
	* ou dclenche une erreur dans les autres cas
	* @param string $req requte SQL avec DELETE
	* @return integer|true retourne une ressource de rsultat ou true sinon dclenche une erreur
	*/
	protected function requeteDelete($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		$resultat = mysql_query($req, $this->cid);

		//au moins une ligne efface?
		if(!mysql_affected_rows($this->cid))
		{
			$this->deconnect();
			trigger_error('Requte avec un delete d\'une ligne non existante requise : '.$req, E_USER_ERROR);
		}
		elseif($resultat === false)
		{
			$numErreur = mysql_errno($this->cid);
			if($numErreur==1217 || $numErreur==1451)
			{
				$this->deconnect();
				trigger_error('Requte errone : une contrainte de cl trangre choue : '.$req, E_USER_ERROR);
			}
			else
			{
				$this->deconnect();
				trigger_error('Requte errone ou droits insuffisants de l\'utilisateur pour l\'effectuer : (erreur n'.$numErreur.') '.$req, E_USER_ERROR);
			}
		}
		return $resultat;
	}

	/**
	* requte avec rcupration de l'id qui vient d'tre cr
	* dans la table qui possde un champ auto-incrment
	* renvoie le numro d'id 0 sinon
	* @param string $req requte SQL avec INSERT
	* @return integer|0 retourne le numro d'id ou 0 sinon dclenche une erreur
	*/
	protected function requeteInsertId($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		if(mysql_query($req, $this->cid) === false)
		{
			$numErreur = mysql_errno($this->cid);
			$this->deconnect();
			trigger_error('Requte errone ou droits insuffisants de l\'utilisateur pour l\'effectuer : (erreur n'.$numErreur.') '.$req, E_USER_ERROR);
		}
		else
			return mysql_insert_id($this->cid);
	}

	/**
	* Indique si la requte renvoie un rsultat
	* retourne false ou true suivant que la requte renvoie un rsultat ou non
	* @param string $req requte SQL avec SELECT
	* @return bool
	*/
	protected function selectPresence($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			mysql_free_result($res);
			$ret = true;
		}
		return $ret;
	}

	/**
	* retourne une variable contenant le rsultat
	* d'une requte qui ne renvoie qu'une valeur (une ligne, une colonne)
	* si vide renvoie false
	* @param string $req requte SQL avec SELECT  un seul champ
	* @return false|array
	*/
	protected function selectSimple($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			$row = mysql_fetch_row($res);
			$ret = stripslashes($row[0]);
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* retourne dans un tableau indic des noms de champs le rsultat du select demand
	* si vide renvoie false
	* @param string $req requte SQL avec SELECT  plusieurs champs
	* @return false|array
	*/
	protected function selectMultiple($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			$i=0;
			while($row = mysql_fetch_assoc($res))
			{
				foreach($row as $k => $v)
				{
					$ret["$k"][$i] = stripslashes($v);
				}
				$i++;
			}
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* retourne dans un tableau indic des noms de champs le rsultat du select demand
	* les valeurs de la premire colonne servent  indicer les autres
	* si vide renvoie false
	* pratique notamment dans le cas de cases  cocher multiples
	* @param string $req requte SQL avec SELECT  plusieurs champs
	* @return false|array
	*/
	protected function selectMultipleCombine($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			while($row = mysql_fetch_assoc($res))
			{
				$indice = $row[key($row)];
                foreach($row as $k => $v)
				{
					$ret["$k"][$indice] = stripslashes($v);
				}
			}
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* mme que selectMultipleCombine mais l'indice de ligne est la premire
	* dimension du tableau retourn
	* retourne dans un tableau indic des noms de champs le rsultat du select demand
	* les valeurs de la premire colonne servent  indicer les autres
	* si vide renvoie false
	* pratique notamment dans le cas de cases  cocher multiples
	* @param string $req requte SQL avec SELECT  plusieurs champs
	* @return false|array
	*/
	protected function selectMultipleCombineIndPrem($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			while($row = mysql_fetch_assoc($res))
			{
				$indice = $row[key($row)];
                foreach($row as $k => $v)
				{
					$ret[$indice]["$k"] = stripslashes($v);
				}
			}
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* mme que selectMultiple mais retourne un tableau
	* avec pour seule dimension l'indice numrique
	* de ligne de rsultat, pas de nom de champ,
	* une seule colonne de rsultat
	* si vide renvoie false
	* @param string $req requte SQL avec SELECT  un seul champ
	* @return false|array
	*/
	protected function selectMultipleUneColonne($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			$i=0;
			while($row = mysql_fetch_row($res))
			{
				$ret[$i] = stripslashes($row[0]);
				$i++;
			}
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* mme que selectMultiple mais retourne un tableau
	* avec les noms des champs pour seule dimension
	* pas d'indice numrique,
	* on ne rcupre que la premire ligne de rsultat
	* si vide renvoie false
	* @param string $req requte SQL avec SELECT  plusieurs champs
	* @return false|array
	*/
	protected function selectMultipleUneLigne($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			$ret = mysql_fetch_assoc($res);
			mysql_free_result($res);
			foreach($ret as $k => $v)
			{
				$ret["$k"] = stripslashes($v);
			}
		}
		return $ret;
	}

	/**
	* mme que selectMultiple mais l'indice de ligne est la premire
	* dimension du tableau retourn
	* si vide renvoie false
	* @param string $req requte SQL avec SELECT  plusieurs champs
	* @return false|array
	*/
	protected function selectMultipleIndPrem($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			$i=0;
			while($row = mysql_fetch_assoc($res))
			{
				foreach($row as $k => $v)
				{
					$ret[$i]["$k"] = stripslashes($v);
				}
				$i++;
			}
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* mme que selectMultiple mais les noms de champs
	* sont remplacs par les numros de colonne
	* si vide renvoie false
	* @param string $req requte SQL avec SELECT  plusieurs champs
	* @return false|array
	*/
	protected function selectMultipleNum($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			$i=0;
			while($row = mysql_fetch_row($res))
			{
				for($j=0;$j<mysql_num_fields($res);$j++)
				{
					$ret[$j][$i] = stripslashes($row[$j]);
				}
				$i++;
			}
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* mme que selectMultipleIndPrem mais les noms de champs
	* sont remplacs par les numros de colonne
	* si vide renvoie false
	* @param string $req requte SQL avec SELECT  plusieurs champs
	* @return false|array
	*/
	protected function selectMultipleIndPremNum($req)
	{
		$ret = false;
		$res = $this->requete($req);
		if($res && mysql_num_rows($res))
		{
			$i=0;
			while($row = mysql_fetch_row($res))
			{
				for($j=0;$j<mysql_num_fields($res);$j++)
				{
					$ret[$i][$j] = stripslashes($row[$j]);
				}
				$i++;
			}
			mysql_free_result($res);
		}
		return $ret;
	}

	/**
	* indique si la dernire requte concernait au moins une ligne prsente
	* plus strict que mysql_affected_rows()
	* @param integer $linkid identifiant de connexion au serveur mysql
	* @return bool
	*/
	protected function mysql_matched_rows($linkid = null)
	{
		 $linkid? $strInfo = mysql_info($linkid) : $strInfo = mysql_info();

		 $return = true;
		 ereg("Rows matched: ([0-9]*)", $strInfo, $rows_matched);

		 $return= (bool)$rows_matched[1];

		 return $return;
	}

//////////////////////gestion du transactionnel


	/**
	* BEGIN pour dmarrer une transaction
	*/
	protected function transDebut()
	{
		$this->requete('BEGIN');
	}

	/**
	* requte transactionnelle autre
	* si requte invalide transaction invalide : roollback et dclenche une erreur
	* @param string $req requte SQL
	*/
	protected function transReq($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		$reqValide = mysql_query($req, $this->cid);

		//stocke la requte qui pose problme pour le message d'erreur
		if($reqValide === false)
			$this->reqEnErreur .= '<br/>(erreur n'.mysql_errno($this->cid).') '.$req;

		$this->transValid *= $reqValide;
	}

	/**
	* requte transactionnelle UPDATE
	*  utiliser si l'update concerne au moins
	* une ligne obligatoirement prsente
	* si aucune ligne  mettre  jour : transaction invalide : roollback et dclenche une erreur
	* @param string $req requte SQL avec UPDATE
	*/
	protected function transReqUpdate($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		$reqValide = mysql_query($req, $this->cid);

		//au moins une ligne prsente?
		if(!$this->mysql_matched_rows($this->cid))
		{
			$reqValide = false;
			$this->reqEnErreur .= "<br/>update d'une ligne obligatoire non prsente :<br/>".$req;
		}
		//pb contrainte de cl?
		elseif($reqValide === false)
		{
			$numErreur = mysql_errno($this->cid);
			if($numErreur==1216 || $numErreur==1217 || $numErreur==1451 || $numErreur==1452)
				$this->reqEnErreur .= "<br/>une contrainte de cl trangre choue :<br/>".$req;
			else
				//stocke la requte qui pose problme pour le message d'erreur
				$this->reqEnErreur .= '<br/>(erreur n'.$numErreur.') '.$req;
		}

		$this->transValid *= $reqValide;
	}

	/**
	* requte transactionnelle DELETE
	*  utiliser si le delete concerne au moins
	* une ligne obligatoirement prsente
	* si aucune ligne  effacer : transaction invalide : roollback et dclenche une erreur
	* @param string $req requte SQL avec DELETE
	*/
	protected function transReqDelete($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		$reqValide = mysql_query($req, $this->cid);

		//au moins une ligne efface?
		if(!mysql_affected_rows($this->cid))
		{
			$reqValide = false;
			$this->reqEnErreur .= "<br/>delete d'une ligne obligatoire non prsente :<br/>".$req;
		}
		//pb contrainte de cl?
		elseif($reqValide === false)
		{
			$numErreur = mysql_errno($this->cid);
			if($numErreur==1217 || $numErreur==1451)
				$this->reqEnErreur .= "<br/>une contrainte de cl trangre choue :<br/>".$req;
			else
				//stocke la requte qui pose problme pour le message d'erreur
				$this->reqEnErreur .= '<br/>(erreur n'.$numErreur.') '.$req;
		}

		$this->transValid *= $reqValide;
	}

	/**
	* requte transactionnelle avec renvoi
	* de l'id autoincrment pour INSERT
	* si requte invalide transaction invalide : roollback et dclenche une erreur
	* @param string $req requte SQL avec INSERT
	* @return integer|0 retourne le numro d'id ou 0 sinon dclenche une erreur
	*/
	protected function transReqInsertId($req)
	{
		if($this->debug)
			$this->trace->ajoute($req);

		$reqValide = mysql_query($req, $this->cid);

		//stocke la requte qui pose problme pour le message d'erreur
		if($reqValide === false)
		{
			$this->reqEnErreur .= '<br/>(erreur n'.mysql_errno($this->cid).') '.$req;
			$id = false;
		}
		else
		{
			$id = mysql_insert_id($this->cid);
		}

		$this->transValid *= $reqValide;

		//indique qu'un id auto-incrment n'a pas t rcupr
		if(!$id)
			$this->reqEnErreur .= '<br/>id auto-incrment non rcupr';

		$this->transValid *= (bool) $id;
		return $id;
	}

	/**
	* COMMIT ou ROLLBACK
	* renvoie true si la transaction s'est correctemment droule
	* false sinon
	* @return bool
	*/
	protected function transFin()
	{
		if($this->transValid)
		{
			$this->requete('COMMIT');
			return true;
		}
		else
		{
			$this->requete('ROLLBACK');
			$this->transValid = true;
			trigger_error('Une des requtes n\'a pas abouti, un ROLLBACK a t effectu : '.$this->reqEnErreur, E_USER_ERROR);
			return false;
		}
	}
}

?>