<?php
/**
 * Piwik - Open source web analytics
 * 
 * @link http://piwik.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html Gpl v3 or later
 * @version $Id: API.php 518 2008-06-09 00:44:10Z matt $
 * 
 * @package Piwik_Provenance
 */

/**
 * 
 * @package Piwik_Provenance
 */
class Piwik_Provenance_API extends Piwik_Apiable
{
	static private $instance = null;
	
	static public function getInstance()
	{
		if (self::$instance == null)
		{            
			$c = __CLASS__;
			self::$instance = new $c();
		}
		return self::$instance;
	}

	public function getProvenance( $idSite, $period, $date )
	{
		Piwik::checkUserHasViewAccess( $idSite );
		$archive = Piwik_Archive::build($idSite, $period, $date );
		$dataTable = $archive->getDataTable('Provenance_hostnameExt');
		$dataTable->queueFilter('Piwik_DataTable_Filter_ColumnCallbackAddMetadata', array('label', 'url', 'Piwik_getHostcelUrl'));
		//$dataTable->queueFilter('Piwik_DataTable_Filter_ColumnCallbackReplace', array('label', 'Piwik_getHostcelName'));
		$dataTable->queueFilter('Piwik_DataTable_Filter_ReplaceColumnNames');
		return $dataTable;
	}
	
	/**
	 * Example of getting a RAW BLOB
	 *
	 * @return blob
	 */
	public function getProvenanceBlob( $idSite, $period, $date )
	{
		Piwik::checkUserHasViewAccess( $idSite );
		$archive = Piwik_Archive::build($idSite, $period, $date );
		$dataTable = $archive->getBlob('Provenance_hostnameExt');
		return $dataTable;
	}
}


function Piwik_getHostcelName($in)
{
	if(empty($in))
	{
		return Piwik_Translate('General_Unknown');
	}
	elseif(strtolower($in) === 'ip')
	{
		return "IP";
	}
	else
	{
		return ucfirst(substr($in, 0, strpos($in, '.')));
	}
	
}


function Piwik_getHostcelUrl($in)
{
	if(empty($in)
		|| strtolower($in) === 'ip')
	{
		return "http://piwik.org/";
	}
	else
	{
		return "http://www.".$in."/";
	}
}
