<?php
/**
 * Piwik - Open source web analytics
 * 
 * @link http://piwik.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html Gpl v3 or later
 * @version $Id: Provenance.php 526 2008-06-25 23:57:04Z matt $
 * 
 * @package Piwik_Provenance
 */
	
/**
 * 
 * @package Piwik_Provenance
 */
class Piwik_Provenance extends Piwik_Plugin
{
	public function getInformation()
	{
		$info = array(
			'name' => 'Provenance',
			'description' => 'Provenance // lookup during logging + archive + display',
			'author' => 'Piwik',
			'homepage' => 'http://piwik.org/',
			'version' => '0.1',
			'LogStatsPlugin' => true, // this plugin must be loaded during the stats logging
		);
		
		return $info;
	}
	
	function install()
	{
		// add column hostname / hostname ext in the visit table
		$query = "ALTER IGNORE TABLE `".Piwik::prefixTable('log_visit')."` ADD `cel_provenance` VARCHAR( 30 ) NULL";
		
		// if the column already exist do not throw error. Could be installed twice...
		try {
			Zend_Registry::get('db')->query($query);
		}
		catch(Exception $e){}
	}
	
	function uninstall()
	{
		// add column hostname / hostname ext in the visit table
		$query = "ALTER TABLE `".Piwik::prefixTable('log_visit')."` DROP `cel_provenance`";
		Zend_Registry::get('db')->query($query);
	}
	
	function postLoad()
	{
		// when the plugin is loaded during LogStats these functions are not defined
		if(function_exists('Piwik_AddWidget'))
		{
			Piwik_AddWidget( 'Provenance', 'getProvenance', Piwik_Translate('Provenance_WidgetProvenances'));
			Piwik_AddMenu('Visitors', Piwik_Translate('UserSettings_SubmenuSettings'), array('module' => 'UserSettings'));
			//Piwik_RenameMenuEntry('Visitors', Piwik_Translate('UserCountry_SubmenuLocations'), 
			//						'Visitors', Piwik_Translate('Provenance_SubmenuLocationsProvenance'));
			
			//Piwik_AddAction('template_headerUserCountry', array('Piwik_Provenance','headerUserCountry'));
			//Piwik_AddAction('template_footerUserCountry', array('Piwik_Provenance','footerUserCountry'));
		}		
	}
	
	function getListHooksRegistered()
	{
		$hooks = array(
			'ArchiveProcessing_Day.compute' => 'archiveDay',
			'ArchiveProcessing_Period.compute' => 'archiveMonth',
			'LogStats.newVisitorInformation' => 'logProvenanceInfo',
		);
		return $hooks;
	}
	
	
	function archiveMonth( $notification )
	{
		$archiveProcessing = $notification->getNotificationObject();
		
		$dataTableToSum = array( 
				'Provenance_hostnameExt',
		);
		
		$archiveProcessing->archiveDataTable($dataTableToSum);
	}
		
	/**
	 * Archive the Provenance count
	 */
	function archiveDay($notification)
	{
		$archiveProcessing = $notification->getNotificationObject();
		
		$recordName = 'Provenance_hostnameExt';
		$labelSQL = "cel_provenance";
		$tableProvenance = $archiveProcessing->getDataTableInterestForLabel($labelSQL);
		$record = new Piwik_ArchiveProcessing_Record_BlobArray($recordName, $tableProvenance->getSerialized());
		
		//echo $tableProvenance;
		//Piwik::printMemoryUsage("End of ".get_class($this)." "); 

	}
	
	/**
	 * Logs the Provenance in the log_visit table
	 */
	public function logProvenanceInfo($notification)
	{
		$visitorInfo =& $notification->getNotificationObject();
		
		//$hostname = $this->getHost($visitorInfo['cel_ip']);
		//$hostnameExtension = $this->getHostnameExt($hostname);
		$hostnameprovenance = $this->getHostnameProvenance($visitorInfo['cel_ip']);
		// add the value to save in the table log_visit
		$visitorInfo['cel_provenance'] = $hostnameprovenance;
	}
	
	/**
	 * Returns the hostname extension (site.co.jp in fvae.VARG.ceaga.site.co.jp)
	 * given the full hostname looked up from the IP
	 * 
	 * @param string $hostname
	 * 
	 * @return string
	 */
	
	private function getHostnameProvenance($hostname)
	{
		
			$e = explode('.', $hostname);
			$s = sizeof($e);
			
			// if extension not correct
			if(($e[0]=="172"&&$e[1]=="16")||($e[0]=="10"&&($e[1]=="77"||$e[1]=="93"||$e[1]=="94")))
			{
				return 'interne';
			}
			else
			{
				return 'externe';
			}
		
	}
	/**
	 * Returns the hostname given the string IP in the format ip2long
	 * php.net/ip2long
	 * 
	 * @param string $ip
	 * 
	 * @return string hostname
	 */
	private function getHost($ip)
	{
		return trim(strtolower(@gethostbyaddr(long2ip($ip))));
	}

	public function headerUserCountry($notification)
	{
		$out =& $notification->getNotificationObject();
		$out = '<div id="leftcolumn">';
	}
	public function footerUserCountry($notification)
	{
		$out =& $notification->getNotificationObject();
		$out = '</div>
			<div id="rightcolumn">
			<h2>Provenances</h2>';
		$out .= Piwik_FrontController::getInstance()->fetchDispatch('Provenance','getProvenance');
		$out .= '</div>';
	}
}

