<?php
/**
 * Basik Framework
 *
 * LICENCE: CeCILL
 *
 * @copyright 2007 : Aurlien Rich
 * @license http://www.cecill.info
 * @version 1.0
 * @link
 * @since
 *
 * aurelien.riche@yahoo.fr
 *
 * Ce logiciel est un programme informatique (framework en PHP 5) servant
 *  simplifier et automatiser des traitements PHP rcurrents tout en
 * se conformant  la mthodologie M.V.C. et tout en gardant la matrise du code.
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe 
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement,
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit.
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
 */


/**
 * Trace
 *
 * Trace l'xcution de l'application
 * (Singleton)
 *
 * @copyright 2007 : Aurlien Rich
 * @license http://www.cecill.info
 * @version
 * @link
 * @since
 */
class Trace
{
	/**
	* instance de la classe
	* @var Trace
	*/
	private static $instance;

	/**
	* contient tous les lments de la trace qui seront affichs
	* @var array
	*/
	private $laTrace = array();

	/**
	* contient tous les lments du mode verbeux qui seront affichs
	* @var array
	*/
	private $laTraceV = array();

	/**
	* paramtre d'activation du traage
	* @var bool
	*/
	private $actif = 0;

	/**
	* paramtre d'activation du mode verbeux, droulement du M.V.C.
	* @var bool
	*/
	private $actifV = 0;

	/**
	* Constructeur priv ; empche la cration directe d'objet
	*/
	private function __construct()
	{

	}

	/**
	* Mthode singleton
	* @return Trace
	*/
	public static function singleton()
	{
		if(!isset(self::$instance))
		{
			$c = __CLASS__;
			self::$instance = new $c;
		}
		return self::$instance;
	}

	/**
	* Activation du traage ou non
	* @param bool $actif
	*/
	public function active($actif)
	{
		$this->actif = $actif;
	}

	/**
	* Activation du mode verbeux ou non
	* @param bool $actif
	*/
	public function activeV($actif)
	{
		$this->actifV = $actif;
	}

	/**
	* Ajoute une trace de l'xcution de l'endroit o on l'appelle
	* - si pas d'arguments : trace par dfaut
	* - si argument(s) : ajout dans le tableau laTrace
	* @param mixed nombre et type des arguments libres
	*/
	public function ajoute()
	{
		if($this->actif)
		{
			$ind = count($this->laTrace);
			if(func_num_args()==0)
			{
				$bt = debug_backtrace();
				$this->laTrace[$ind]['tfichier'] = $bt[0]['file'];
				$this->laTrace[$ind]['tligne'] = $bt[0]['line'];
				$i = 1;
				while(isset($bt[$i]))
				{
					$this->laTrace[$ind][$i-1]['fichier'] = $bt[$i]['file'];
					$this->laTrace[$ind][$i-1]['ligne'] = $bt[$i]['line'];
					if(isset($bt[$i]['class']))
						$this->laTrace[$ind][$i-1]['class'] = $bt[$i]['class'];
					if(isset($bt[$i]['type']))
					{
						switch($bt[$i]['type'])
						{
							case '->':
								$this->laTrace[$ind][$i-1]['type'] = 'mthode de la classe';
								break;
							case '::':
								$this->laTrace[$ind][$i-1]['type'] = 'mthode statique de la classe';
								break;
							default:
								$this->laTrace[$ind][$i-1]['type'] = 'fonction';
						}
					}
					if(isset($bt[$i]['function']))
						$this->laTrace[$ind][$i-1]['fonction'] = $bt[$i]['function'];
					if(isset($bt[$i]['args']) && !empty($bt[$i]['args']))
						$this->laTrace[$ind][$i-1]['arguments'] = $bt[$i]['args'];
					$i++;
				}
			}
			else
			{
				for($i=0;$i<func_num_args();$i++)
				{
					$this->laTrace[$ind][] = func_get_arg($i);
				}
			}
		}
	}

	/**
	* ajoute une trace du droulement M.V.C.
	* @param string nombre des arguments libre
	*/
	public function ajouteV()
	{
		if($this->actifV)
		{
			$ind = count($this->laTraceV);

			for($i=0;$i<func_num_args();$i++)
			{
				$this->laTraceV[$ind][] = func_get_arg($i);
			}
		}
	}

	/**
	* Fonction utilisateur pour afficher le code html non interprt
	* prsent dans la trace
	* @param mixed
	*/
	private function html_decode(&$item)
	{
		if(is_string($item))
			$item = htmlentities($item);

	}

	/**
	* Affiche la trace
	* @param bool $plus si true : utilisation de var_dump au lieu de print_r
	*/
	public function afficheTrace($plus = false)
	{
		if($this->actifV || $this->actif)
		{
			echo '<div style="clear: both; position: absolute; left: 0px; text-align: left; background: White;">';
		}
		
		if($this->actifV)
		{
			echo '<div style="position: relative; color: Green; font-family: monospace; font-size: 8pt;">';
			echo '#############################<br />***** DEROULEMENT M.V.C. ****<br />#############################<br /><br />';
			for($i=0;$i<count($this->laTraceV);$i++)
			{
				for($j=0;$j<count($this->laTraceV[$i]);$j++)
				{
					echo $this->laTraceV[$i][$j];
				}
				echo '<br /><br />';
			}
			echo '<br /><br /></div>';
			//vide la trace
			$this->laTraceV = array();
		}

		if($this->actif)
		{
			//dcode l'html prsent
			array_walk_recursive($this->laTrace,array('Trace','html_decode'));

			//choix du rendu
			if($plus)
				$rendu = 'var_dump';
			else
				$rendu = 'print_r';
			echo '<div style="position: relative; color: Maroon; font-family: monospace; font-size: 8pt;">';
			echo '#############################<br />*********** TRACE ***********<br />#############################<br /><br />';
			for($i=0;$i<count($this->laTrace);$i++)
			{
				if(isset($this->laTrace[$i]['tfichier']))
				{
					echo '<em>==== Trace du fichier <b>'.$this->laTrace[$i]['tfichier'].'</b>  la ligne <b>'.$this->laTrace[$i]['tligne'].'</b> ====</em><br /><br />';
					for($j=0;$j<(count($this->laTrace[$i])-2);$j++)
					{
						echo '<u>Fichier</u> : <b>'.$this->laTrace[$i][$j]['fichier'].'</b>    <u>ligne</u> : <b>'.$this->laTrace[$i][$j]['ligne'].'</b><br />';
						if(isset($this->laTrace[$i][$j]['class']))
							echo '<u>Classe</u> : <b>'.$this->laTrace[$i][$j]['class'].'</b><br />';
						if(isset($this->laTrace[$i][$j]['type']))
							echo '<u>Type d\'appel</u> : <b>'.$this->laTrace[$i][$j]['type'].'</b><br />';
						if(isset($this->laTrace[$i][$j]['fonction']))
							echo '<u>Fonction</u> : <b>'.$this->laTrace[$i][$j]['fonction'].'</b><br />';
						if(isset($this->laTrace[$i][$j]['arguments']))
						{
							echo '<u>Arguments</u> :<br />';
							echo '<pre><b>';
							$rendu($this->laTrace[$i][$j]['arguments']);
							echo '</b></pre>';
						}
						echo '__________<br /><br />';
					}
				}
				else
				{
					for($j=0;$j<count($this->laTrace[$i]);$j++)
					{
						echo '<pre><b>';
						$rendu($this->laTrace[$i][$j]);
						echo '</b></pre>';
					}
				}
				echo '<br />*********************************<br /><br />';
			}
			echo '</div>';
			//vide la trace
			$this->laTrace = array();
		}
		
		if($this->actifV || $this->actif)
		{
			echo '</div>';
		}
	}
}
?>